var HttpListener = importNamespace('System.Net').HttpListener;
var Encoding = importNamespace('System.Text').Encoding;
var System = importNamespace("System");
var IO = importNamespace('System.IO');
var File = IO.File;
var Directory = IO.Directory;


// 默认配置
let config = {
    auth: [""],
    port: 61545,
};

// 配置文件路径
const CONFIG_DIR = 'Serein/plugins/BackupJS';
const CONFIG_FILE = `${CONFIG_DIR}/config.json`;

// 生成随机密钥
function generateRandomKey(length = 12) {
    const chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    return Array.from({ length }, () => chars.charAt(Math.floor(Math.random() * chars.length))).join('');
}

// 加载或创建配置文件
function loadConfig() {
    if (Directory.Exists(CONFIG_DIR) && File.Exists(CONFIG_FILE)) {
        try {
            const loadedConfig = JSON.parse(File.ReadAllText(CONFIG_FILE));
            if (!loadedConfig.auth || !loadedConfig.auth[0]) {
                loadedConfig.auth = [generateRandomKey()];
                saveConfig(loadedConfig);
                serein.console.warn(`AuthKey 为空，已生成随机密钥：${loadedConfig.auth[0]}`);
            }
            return loadedConfig;
        } catch (e) {
            throw new Error(`读取配置文件时出现问题：${e.message}`);
        }
    } else {
        Directory.CreateDirectory(CONFIG_DIR);
        config.auth = [generateRandomKey()];
        saveConfig(config);
        serein.console.warn("配置文件不存在，已重新创建默认配置");
        serein.console.warn(`随机生成的 AuthKey 为：${config.auth[0]}`);
        return config;
    }
}

// 保存配置文件
function saveConfig(config) {
    File.WriteAllText(CONFIG_FILE, JSON.stringify(config, null, 4));
}


let listener = null;


serein.setListener('PluginsLoaded', () => {
    try {
        // 加载配置
        config = loadConfig();
        serein.console.log("配置已加载");

        // 初始化 HttpListener
        if (!listener) {
            listener = new HttpListener();
            listener.Prefixes.Add(`http://127.0.0.1:${config.port}/serein/`);
            listener.Start();
            startListener();
            serein.console.log(`服务已启动，正在监听 http://127.0.0.1:${config.port}/serein/`);
        } else {
            serein.console.warn("HttpListener 已经初始化，无需重复启动");
        }
    } catch (e) {
        serein.console.error(`插件加载时出错：${e.message}`);
    }
});


// 设置卸载回调，释放资源
serein.setListener('PluginsUnloading', () => {
    if (listener) {
        try {
            listener.Close();
            listener.Dispose();
            //serein.console.log("HttpListener 已释放");
        } catch (e) {
            serein.console.error(`释放 HttpListener 时出错：${e.message}`);
        }
    }
});

// 主监听逻辑（异步）
function startListener() {
    try {
        listener.BeginGetContext(processRequest, null); // 开始异步监听
    } catch (e) {
        serein.console.error(`启动异步监听时出错：${e.message}`);
    }
}

// 处理请求
function processRequest(result) {
    if (!listener.IsListening) return;

    try {
        const context = listener.EndGetContext(result);
        serein.console.log(`收到请求：${context.Request.RawUrl} 来自 IP：${context.Request.RemoteEndPoint.Address}`);
        const response = handleRequest(context.Request.RawUrl, context.Request.Headers["Authorization"]);
        sendResponse(context, response);
        listener.BeginGetContext(processRequest, null); // 继续异步监听
    } catch (e) {
        serein.console.error(`处理请求时出现错误：${e.message}`);
    }
}

// 构建响应
function sendResponse(context, response) {
    const responseBytes = Encoding.UTF8.GetBytes(response.responseBody);
    context.Response.StatusCode = response.statusCode;
    context.Response.StatusDescription = response.statusDescription;
    context.Response.ContentLength64 = responseBytes.length;
    context.Response.OutputStream.Write(responseBytes, 0, responseBytes.length);
    context.Response.Close();
}

// 请求处理逻辑
function handleRequest(rawUrl, authHeader) {
    if (!isValidAuthKey(authHeader)) {
        return createErrorResponse(401, "Unauthorized: Invalid or missing Authorization key");
    }

    if (rawUrl === '/serein/health') {
        return createSuccessResponse(200, "OK");
    }

    if (rawUrl.startsWith('/serein/start')) {
        return handleServerAction(rawUrl, "start");
    }

    if (rawUrl.startsWith('/serein/stop')) {
        return handleServerAction(rawUrl, "stop");
    }

    if (rawUrl.startsWith('/serein/terminate')) {
        return handleServerAction(rawUrl, "terminate");
    }

    return createErrorResponse(404, "Not Found");
}

// 处理服务器操作
function handleServerAction(rawUrl, action) {
    const { id, pmid, gmid, msg } = getQueryParams(rawUrl);
    if (!id) {
        return createErrorResponse(400, "Missing required parameter: id");
    }

    try {
        serein.servers[id][action]();
        return sendMessageAfterAction(id, pmid, gmid, msg, `Server ${id} ${action}ed successfully`);
    } catch (e) {
        return createErrorResponse(500, `Failed to ${action} server ${id}: ${e.message}`);
    }
}

// 发送消息
// 发送消息
function sendMessageAfterAction(id, pmid, gmid, message, successMessage) {
    // 如果消息是 Base64 编码的，则解码它
    if (isBase64(message)) {
        message = decodeBase64(message);
    } else {
        // 否则解码 URL 编码的字符串
        message = decodeURIComponent(message);
    }

    if (pmid && message) {
        try {
            serein.connection.sendPrivateMsg(pmid, message);
            successMessage += ` and private message sent to ${pmid}`;
        } catch (e) {
            return createErrorResponse(500, `Failed to send private message to user ${pmid}: ${e.message}`);
        }
    }

    if (gmid && message) {
        try {
            serein.connection.sendGroupMsg(gmid, message);
            successMessage += ` and group message sent to group ${gmid}`;
        } catch (e) {
            return createErrorResponse(500, `Failed to send group message to group ${gmid}: ${e.message}`);
        }
    }

    return createSuccessResponse(200, successMessage);
}

// 检查是否为 Base64 编码
function isBase64(str) {
    try {
        // 使用正则检查字符串是否符合 Base64 格式
        return btoa(atob(str)) === str;
    } catch (err) {
        return false;
    }
}


function isBase64(str) {
    try {
        // 调用 C# 的方法尝试解码
        var decoded = System.Convert.FromBase64String(str);

        // 可选：检查解码后的字节内容
        console.log("Decoded byte length:", decoded.Length);

        return true;
    } catch (e) {
        return false; // 解码失败
    }
}

// 解码 Base64 字符串为原始数据
function decodeBase64(str) {
    var decoded = System.Convert.FromBase64String(str);
    return System.Text.Encoding.UTF8.GetString(decoded); // 转换为 UTF-8 字符串
}


// 验证授权密钥
function isValidAuthKey(authHeader) {
    return authHeader && authHeader.startsWith("Bearer ") && config.auth.includes(authHeader.substring(7));
}

// 创建成功响应
function createSuccessResponse(statusCode, message) {
    return {
        statusCode,
        statusDescription: "OK",
        responseBody: JSON.stringify({ statusCode, message }),
    };
}

// 创建错误响应
function createErrorResponse(statusCode, message) {
    return {
        statusCode,
        statusDescription: statusCode === 401 ? "Unauthorized" : "Error",
        responseBody: JSON.stringify({ statusCode, message }),
    };
}

// 获取查询参数
function getQueryParams(url) {
    const query = url.split('?')[1];
    if (!query) return {};
    return Object.fromEntries(query.split('&').map(param => param.split('=').map(decodeURIComponent)));
}
